import { Express }               from 'express-serve-static-core';
import express                   from 'express';
import RoutesManager from '../express/RoutesManager';
import SecurityMiddleware from '../middlewares/SecurityMiddleware';
import SecurityCheckType from '../types/SecurityCheckType';
import db from '../helpers/DatabaseHelper';
// import { User, UserRole } from '@prisma/client';
import { StatusCodes } from 'http-status-codes';

class UserRoutes implements RoutesManager {
     registerOnBackend(backend: Express): void {
          backend.get('/users', SecurityMiddleware.check(true, SecurityCheckType.ADMIN), this.getUsers.bind(this));
          backend.patch('/users/:userId/role', SecurityMiddleware.check(true, SecurityCheckType.ADMIN), this.changeRole.bind(this))
          backend.get('/users/:userId/assignments', SecurityMiddleware.check(true, SecurityCheckType.TEACHING_STAFF), this.getUsersAssignments.bind(this));
     }
     
     private async getUsers(req: express.Request, res: express.Response) {          
          const users = await db.user.findMany();
          return req.session.sendResponse(res, StatusCodes.OK, users);
     }
     
     private async getUsersAssignments(req: express.Request, res: express.Response) {
          const id = +req.params.userId;
          const user = await db.user.findUnique({
               where: { 
                    id: id,
               },
               include: {
                    assignments: true,

               }, // Include the assignments related to the user
          });
          return req.session.sendResponse(res, StatusCodes.OK, user);    
     }
     
     private async changeRole(req: express.Request, res: express.Response) {
          const id = +req.params.userId;
          const newRole = req.body.newRole;
          // check admin
          
          if (id != req.session.profile.id) {
               
               await db.user.update({
                    where : {
                         id : id
                    },
                    data : {
                         role: newRole
                    }
               });
               req.session.sendResponse(res, StatusCodes.OK);
          } else {
               req.session.sendResponse(res, StatusCodes.FORBIDDEN);
          }
     }
}

export default new UserRoutes();
